<?php
/**
 * @file
 * Drop Jobs profile.
 */

/**
 * A trick to enforce page refresh when theme is changed from an overlay.
 */
function drop_jobs_admin_paths_alter(&$paths) {
  $paths['admin/appearance/default*'] = FALSE;
}

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Allows the profile to alter the site configuration form.
 */
function drop_jobs_form_install_configure_form_alter(&$form, $form_state) {
  // Many modules set messages during installation that are very annoying.
  // (I'm looking at you Date and IMCE)
  // Lets remove these and read the only message that should be set.
  drupal_get_messages('status');
  drupal_get_messages('warning');

  // Warn about settings.php permissions risk
  $settings_dir = conf_path();
  $settings_file = $settings_dir . '/settings.php';
  // Check that $_POST is empty so we only show this message when the form is
  // first displayed, not on the next page after it is submitted. (We do not
  // want to repeat it multiple times because it is a general warning that is
  // not related to the rest of the installation process; it would also be
  // especially out of place on the last page of the installer, where it would
  // distract from the message that the Drupal installation has completed
  // successfully.)
  if (empty($_POST) && (!drupal_verify_install_file(DRUPAL_ROOT . '/' . $settings_file, FILE_EXIST | FILE_READABLE | FILE_NOT_WRITABLE) || !drupal_verify_install_file(DRUPAL_ROOT . '/' . $settings_dir, FILE_NOT_WRITABLE, 'dir'))) {
    drupal_set_message(st('All necessary changes to %dir and %file have been made, so you should remove write permissions to them now in order to avoid security risks. If you are unsure how to do so, consult the <a href="@handbook_url">online handbook</a>.', array('%dir' => $settings_dir, '%file' => $settings_file, '@handbook_url' => 'http://drupal.org/server-permissions')), 'warning');
  }

  // Pre-populate some fields.
  $form['site_information']['site_name']['#default_value'] = 'Drop Jobs'; // We don't use t() intentionally.
  $form['site_information']['site_mail']['#default_value'] = 'admin@' . $_SERVER['HTTP_HOST'];
  $form['admin_account']['account']['name']['#default_value'] = 'admin';
  $form['admin_account']['account']['mail']['#default_value'] = 'admin@' . $_SERVER['HTTP_HOST'];

  // Add checkbox for example content.
  $form['drop_jobs'] = array(
    '#type' => 'fieldset',
    '#collapsible' => FALSE,
    '#title' => t('Drop Jobs'),
  );

  $form['drop_jobs']['drop_jobs_demo_content'] = array(
    '#type' => 'checkbox',
    '#title' => t('Install demo content'),
    '#description' => t('Check this option to enable demonstration content for Drop Jobs to get your site up and running quickly.'),
    '#default_value' => TRUE,
  );

  $form['#submit'][] = 'drop_jobs_install_configure_form_submit';
}

/**
 * Submit callback.
 *
 * Installs Drop Jobs demonstration content.
 */
function drop_jobs_install_configure_form_submit(&$form, &$form_state) {
  if ($form_state['values']['drop_jobs_demo_content']) {
    // @TODO remove this once http://drupal.org/node/1231302 gets fixed.
    variable_set('node_content_enabled', 1);
    $module = array('drop_jobs_demo');
    module_enable($module);

    // For some reason the demo content only works at this stage when
    // we disable and re-enable the module.
    module_disable($module);
    module_enable($module);

    // Set frontpage to "Welcome" node.
    // First find the nid (default_content is unpredictable like this).
    $nid =
      db_select('node', 'n')
      ->fields('n', array('nid'))
      ->condition('type', 'page', '=')
      ->condition('title', 'Welcome to Drop Jobs!', '=')
      ->execute()
      ->fetchField();

    // Set as frontpage.
    variable_set('site_frontpage', "node/{$nid}");

    // Add menu link for "Terms & Conditions" node.
    // Find nid.
    $nid =
      db_select('node', 'n')
      ->fields('n', array('nid'))
      ->condition('type', 'page', '=')
      ->condition('title', 'Terms and Conditions', '=')
      ->execute()
      ->fetchField();

    // Build menu item.
    $item = array(
      'link_path' => "node/{$nid}",
      'link_title' => 'Terms and Conditions',
      'menu_name' => 'menu-secondary-links',
    );

    // Save menu link.
    menu_link_save($item);
  }
}

/**
 * Implements of hook_install_tasks().
 */
function drop_jobs_install_tasks() {
  return array(
    'drop_jobs_import_vocabularies_batch' => array(
      'display_name' => st('Import terms'),
      'display' => TRUE,
      'type' => 'batch',
      'run' => INSTALL_TASK_RUN_IF_NOT_COMPLETED,
    ),
  );
}

/**
 * Defines batch operation for importing terms.
 */
function drop_jobs_import_vocabularies_batch() {
  $batch = array(
    'operations' => array(
      array('drop_jobs_import_vocabularies', array()),
    ),
    'finished' => 'drop_jobs_import_vocabularies_finished',
    'title' => t('Import terms'),
    'init_message' => t('Starting import.'),
    'progress_message' => t('Processed @current out of @total.'),
    'error_message' => t('Example Batch has encountered an error.'),
  );
  return $batch;
}

/**
 * Import batch operation for vocabularies.
 */
function drop_jobs_import_vocabularies(&$context) {
  if (!isset($context['sandbox']['progress'])) {
    $vocs = taxonomy_vocabulary_load_multiple(FALSE);
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = count($vocs);
    $context['sandbox']['vocs'] = $vocs;
  }

  $voc = array_pop($context['sandbox']['vocs']);
  drop_jobs_import_vocabulary($voc, $context);
  ++$context['sandbox']['progress'];

  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Imports terms into a vocabulary using Taxonomy CSV module.
 */
function drop_jobs_import_vocabulary($voc, &$context) {
  // Note: drupal_get_path('profile', 'drop_jobs') didn't work here.
  $import_dir = drupal_get_path('profile', 'drop_jobs') . '/taxonomy_import/';

  // Use Taxonomy CSV to import terms from a file.
  $module_dir = drupal_get_path('module', 'taxonomy_csv');
  require_once("{$module_dir}/import/taxonomy_csv.import.api.inc");

  // Import terms for each voc, where a .csv file exists.
  $filename = $import_dir . $voc->machine_name . '.csv';
  if (!file_exists($filename) || !is_readable($filename)) {
    return;
  }

  // Set options for import.
  $options['enclosure'] = '"';
  $options['vocabulary_target'] = 'existing';
  $options['vocabulary_id'] = $voc->vid;
  $options['check_hierarchy'] = FALSE;
  $options['set_hierarchy'] = 1;
  $options['url'] = $filename;
//  $options['filter_format'] = 'filtered_html';
  taxonomy_csv_import($options);

  $context['results'][] = t('Imported terms for vocabulary @voc_name', array('@voc_name' => $voc->name));
}

/**
 * Prints message after batch is finished.
 */
function drop_jobs_import_vocabularies_finished($success, $results) {
  $message = "";
  if ($success) {
    $message .= theme('item_list', array('items' => $results));
  }
  $message .= t('Import finished');
  drupal_set_message($message);
}

/**
 * Implements hook_block_info().
 */
function drop_jobs_block_info() {
  return array(
    'powered-by' => array(
      'info' => t('Powered by Drop Jobs'),
      'weight' => '10',
      'cache' => DRUPAL_CACHE_GLOBAL,
    ),
  );
}

/**
 * Implements hook_block_view().
 */
function drop_jobs_block_view($delta = '') {
  if ($delta === 'powered-by') {
    return array(
      'subject' => NULL,
      'content' => '<span>' . t('Powered by <a href="@drop_jobs">Drop Jobs</a>, a <a href="@drupal">Drupal</a> distribution by <a href="@webdrop">Webdrop</a>', array('@drop_jobs' => 'http://drupal.org/project/drop_jobs', '@drupal' => 'http://drupal.org', '@webdrop' => 'http://www.webdrop.com.br')) . '</span>',
    );
  }
}