<?php
/**
 * @file
 * Drupal Ipsum helper functions
 */

/**
 * Submit callback.
 *
 * @see drupal_ipsum_form_node_type_form_alter().
 */
function drupal_ipsum_node_type_form_submit(&$form, &$form_state) {
  $type = $form_state['values']['type'];
  $name = "drupal_ipsum_{$type}_enable";

  if (isset($form_state['values'][$name])) {
    $value = $form_state['values'][$name];
    variable_set($name, $value);
  }
}

/**
 * Add Drupal Ipsum checkbox to text fields in a form and add submit callback and javascript.
 *
 * @param array $form
 * @param string|array $field
 * @param string $label
 */
function _drupal_ipsum_form_add_field(&$form, $field, $label = 'field') {
  $access = array_key_exists('#access', $form[$field]);

  if (!$access || $form[$field]['#access']) {
    // add "Fill field with Drupal Ipsum" checkbox
    $item = array(
      '#type' => 'container',
      '#attributes' => array(
        'class' => array('drupal_ipsum_checkbox'),
      ),
      'drupal_ipsum' => array(
        '#type' => 'checkbox',
        '#title' => t('Fill @field field with Drupal Ipsum text', array('@field' => $label)),
        '#description' => t('This option will override the contents of this field'),
        '#name' => "drupal_ipsum_{$field}",
      )
    );
    $item = drupal_render($item);

    if (!array_key_exists('#prefix', $form[$field])) {
      $form[$field]['#prefix'] = '';
    }

    $form[$field]['#prefix'] .= $item;
    $form[$field]['#states'] = array(
      'invisible' => array(
        "input[name=\"drupal_ipsum_{$field}\"]" => array('checked' => TRUE),
      ),
    );
  }
}

/**
 * Validate callback.
 *
 * @see drupal_ipsum_form_node_form_alter().
 * Use form_set_value() to override fields with Drupal Ipsum text
 */
function drupal_ipsum_node_form_validate(&$form, &$form_state) {
  foreach ($form_state['input'] as $k => $v) {
    if (strpos($k, 'drupal_ipsum_') === 0 && $v == 1) {
      $temp = explode('drupal_ipsum_', $k);
      if (count($temp) > 1) {
//        $lang = $form_state['values']['language'];
        $lang = LANGUAGE_NONE;
        $type = $form[$temp[1]][$lang][0]['value']['#type'];

        if ($temp[1] === 'title') {
          // if only title failed validation, clear the error
          $errors = form_get_errors();
          if (count($errors) === 1 && isset($errors['title'])) {
            form_clear_error();
            drupal_get_messages('error');
          }
          form_set_value($form['title'], drupal_ipsum_generate(NULL, DRUPAL_IPSUM_TYPE_WORD, DRUPAL_IPSUM_START_ANY, FALSE), $form_state);
        }
        elseif ($type === 'textfield') {
          form_set_value($form[$temp[1]][$lang][0]['value'], drupal_ipsum_generate(NULL, DRUPAL_IPSUM_TYPE_WORD, DRUPAL_IPSUM_START_ANY, FALSE), $form_state);
        }
        else {
          include_once drupal_get_path('module', 'devel') . '/krumo/class.krumo.php';
          form_set_value($form[$temp[1]][$lang][0]['value'], drupal_ipsum_generate(), $form_state);
        }
      }
    }
  }
}

/**
 * Page callback.
 *
 * @see drupal_ipsum_menu().
 */
function _drupal_ipsum_show_results($hash) {
  $hash = unserialize(base64_decode($hash));
  extract($hash);

  if (!isset($type) || !isset($length) || !isset($start)) {
    drupal_goto('drupal_ipsum/generate');
    exit;
  }

  $text = drupal_ipsum_generate($length, $type, $start);
  $form = drupal_get_form('drupal_ipsum_generate_form', $length, $type, $start);
  $form = drupal_render($form);

  return array(
    '#theme' => 'drupal_ipsum_results',
    '#drupal_ipsum_text' => $text,
    '#drupal_ipsum_form' => $form,
  );
}

/**
 * Form callback.
 *
 * @see drupal_ipsum_menu().
 */
function drupal_ipsum_generate_form($form, &$form_state, $length = NULL, $type = NULL, $start = NULL, $html = NULL) {
  $form = drupal_ipsum_get_form($length, $type, $start, $html);
  $form['#after_build'][] = 'drupal_ipsum_generate_form_after_build';

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Generate!'),
  );

  return $form;
}

/**
 * Helper function.
 * Returns base form for Drupal Ipsum.
 *
 * @param int $length
 * @param int $type
 * @param int $start
 * @param bool $html
 * @return string
 */
function drupal_ipsum_get_form($length = NULL, $type = NULL, $start = NULL, $html = NULL) {
  $form = array();

  $form['drupal_ipsum_type'] = array(
    '#type' => 'radios',
    '#title' => t('Type of text'),
    '#options' => array(
      DRUPAL_IPSUM_TYPE_WORD => t('Words'),
      DRUPAL_IPSUM_TYPE_SENTENCE => t('Sentences'),
      DRUPAL_IPSUM_TYPE_PARAGRAPH => t('Paragraphs'),
    ),
    '#default_value' => isset($type) ? $type : variable_get_value('drupal_ipsum_type'),
    '#weight' => -4,
  );

  $form['drupal_ipsum_html'] = array(
    '#type' => 'checkbox',
    '#title' => t('Wrap paragraphs in @p tags', array('@p' => '<p>')),
    '#default_value' => isset($html) ? $html : variable_get_value('drupal_ipsum_html'),
    '#states' => array(
      'visible' => array(
        'input[name="drupal_ipsum_type"]' => array('value' => DRUPAL_IPSUM_TYPE_PARAGRAPH),
      ),
    ),
    '#weight' => -3,
  );

  $form['drupal_ipsum_length'] = array(
    '#type' => 'textfield',
    '#title' => t('How much text'),
    '#default_value' => isset($length) ? $length : variable_get_value('drupal_ipsum_length'),
    '#weight' => -2,
    '#size' => 2,
  );

  $form['drupal_ipsum_start'] = array(
    '#type' => 'radios',
    '#title' => t('Start with'),
    '#options' => array(
      DRUPAL_IPSUM_START_ANY => t('Anything'),
      DRUPAL_IPSUM_START_LOREM => DRUPAL_IPSUM_TEXT_LOREM . '&hellip;',
      DRUPAL_IPSUM_START_DRUPAL => DRUPAL_IPSUM_TEXT_DRUPAL . '&hellip;',
    ),
    '#default_value' => isset($start) ? $start : variable_get_value('drupal_ipsum_start'),
    '#weight' => -1,
  );

  return $form;
}

/**
 * After build callback.
 *
 * @see drupal_ipsum_generate_form().
 */
function drupal_ipsum_generate_form_after_build($element) {
  drupal_add_css(drupal_get_path('module', 'drupal_ipsum') . '/css/drupal_ipsum.css');
  drupal_add_js(drupal_get_path('module', 'drupal_ipsum') . '/js/drupal_ipsum.js');
  return $element;
}

/**
 * Validate callback.
 *
 * @see drupal_ipsum_generate_form().
 */
function drupal_ipsum_generate_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['drupal_ipsum_length'])) {
    form_set_error('drupal_ipsum_length', t('Quantity must be numeric!'));
  }
}

/**
 * Submit callback.
 *
 * @see drupal_ipsum_generate_form().
 */
function drupal_ipsum_generate_form_submit(&$form, &$form_state) {
  $hash = array(
    'type' => $form_state['values']['drupal_ipsum_type'],
    'length' => $form_state['values']['drupal_ipsum_length'],
    'start' => $form_state['values']['drupal_ipsum_start'],
  );
  $hash = base64_encode(serialize($hash));

  switch ($form_state['values']['drupal_ipsum_type']) {
    case DRUPAL_IPSUM_TYPE_WORD:
      $type_singular = 'word';
      $type_plural = 'words';
      break;
    case DRUPAL_IPSUM_TYPE_SENTENCE:
      $type_singular = 'sentence';
      $type_plural = 'sentences';
      break;
    case DRUPAL_IPSUM_TYPE_PARAGRAPH:
      $type_singular = 'paragraph';
      $type_plural = 'paragraphs';
      break;
  }

  $summary = format_plural($form_state['values']['drupal_ipsum_length'], 'Generated 1 %type_singular of Drupal Ipsum filler text!', 'Generated @count %type_plural of Drupal Ipsum filler text!', array('%type_singular' => $type_singular, '%type_plural' => $type_plural));

  drupal_set_message($summary);
  drupal_goto("drupal_ipsum/generate/{$hash}");
}

/**
 * Generates Drupal-flavoured lorem-ipsum text and returns results.
 * This is where the magic happens!
 *
 * @param int $length
 * @param int $type
 * @param int $start
 * @param bool $markup
 * @return string
 */
function drupal_ipsum_generate($length = NULL, $type = NULL, $start = NULL, $markup = FALSE) {
  // substitute default values if necessary

  if (!isset($length)) {
    $length = variable_get_value('drupal_ipsum_length');
  }

  if (!isset($type)) {
    $type = variable_get_value('drupal_ipsum_type');
  }

  if (!isset($start)) {
    $start = variable_get_value('drupal_ipsum_start');
  }

  // special beginning?
  $starts_with = NULL;
  if ($start == DRUPAL_IPSUM_START_DRUPAL) {
    $starts_with = DRUPAL_IPSUM_TEXT_DRUPAL;
  }
  elseif ($start == DRUPAL_IPSUM_START_LOREM) {
    $starts_with = DRUPAL_IPSUM_TEXT_LOREM;
  }

  // set Drupal Ipsum vocabulary
  $vocabulary = drupal_ipsum_get_vocabulary();
  Foo_Ipsum::setVocabulary($vocabulary);

  // generate text based on type
  switch ($type) {
    case DRUPAL_IPSUM_TYPE_WORD:
      $return = Foo_Ipsum::words($length, $starts_with);
      break;
    case DRUPAL_IPSUM_TYPE_SENTENCE:
      $return = Foo_Ipsum::sentences($length, $starts_with);
      break;
    case DRUPAL_IPSUM_TYPE_PARAGRAPH:
    default:
      $return = Foo_Ipsum::paragraphs($length, $markup, $starts_with);
      break;
  }

  // horray!
  return $return;
}

/**
 * Return array of words to be used in Drupal ipsum text generation.
 *
 * @return array
 */
function drupal_ipsum_get_vocabulary() {
  return array(
    'drupal',
    'hook',
    'module',
    'theme',
    'alter',
    'node',
    'entity',
    'comment',
    'user',
    'taxonomy',
    'term',
    'vocabulary',
    'content',
    'permission',
    'hack',
    'core',
    'contrib',
    'template',
    'menu',
    'profile',
    'behaviors',
    'ahah',
    'ajax',
    'javascript',
    'css',
    'html',
    'markup',
    'form',
    'api',
    'FAPI',
    'install',
    'uninstall',
    'enable',
    'disable',
    'block',
    'help',
    'documentation',
    'RTFM',
    'post',
    'article',
    'webform',
    'captcha',
    'views',
    'panels',
    'context',
    'ctools',
    'cck',
    'features',
    'cron',
    'locale',
    'i18n',
    'community',
    'server',
    'git',
    'commit',
    'push',
    'pull',
    'diff',
    'issue',
    'project',
    'role',
    'field',
    'cache',
    'session',
    'semaphore',
    'bug',
    'major',
    'minor',
    'critical',
    'meta',
    'profile',
    'color',
    'filter',
    'backup',
    'migrate',
    'actions',
    'rules',
    'plugin',
    'book',
    'contact',
    'dashboard',
    'workbench',
    'trigger',
    'simpletest',
    'devel',
    'generate',
    'flag',
    'bundle',
    'token',
    'variable',
    'wysiwyg',
    'jquery',
    'html5',
    'css3',
    'scheduler',
    'date',
    'calendar',
    'overlay',
    'revision',
    'access',
    'registry',
    'router',
    'link',
    'alias',
    'database',
    'query',
    'media query',
    'responsive',
    'xhtml',
  );
}