<?php
/**
 * @file
 * Views field handlers for Subscriptions.
 */

/**
 * A handler to display subscription start and end dates.
 *
 * @ingroup views_field_handlers
 */
class subs_handler_date extends views_handler_field_date {

  function render($values) {
    if ($this->field_alias === 'subs_date_start') {
      if ($values->subs_date_start == 0) {
        return t('None');
      }
    }
    elseif ($this->field_alias === 'subs_date_end') {
      if ($values->subs_date_end == 0) {
        return t('None');
      }
    }

    return parent::render($values);
  }

}

/**
 * A handler to provide a field with a link to view the subscription.
 *
 * @ingroup views_field_handlers
 */
class subs_handler_view extends views_handler_field {

  function construct() {
    parent::construct();
    $this->additional_fields['sid'] = 'sid';
    $this->additional_fields['type'] = 'type';
  }

  function option_definition() {
    $options = parent::option_definition();
    $options['text'] = array('default' => '', 'translatable' => TRUE);
    return $options;
  }

  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['text'] = array(
      '#type' => 'textfield',
      '#title' => t('Text to display'),
      '#default_value' => $this->options['text'],
    );
  }

  function query() {
    $this->ensure_my_table();
    $this->add_additional_fields();
  }

  function render($values) {
    $type = $values->{$this->aliases['type']};
    $sid = $values->{$this->aliases['sid']};

    // Creating a dummy subscription to check access against.
    $dummy = (object) array('sid' => $sid, 'type' => $type);
    if (!subs_access('view', $dummy)) {
      return;
    }

    $text = !empty($this->options['text']) ? $this->options['text'] : t('view');
    return l($text, 'subscription/' . $sid);
  }

}

/**
 * A handler to provide a field with a link to edit the subscription.
 *
 * @ingroup views_field_handlers
 */
class subs_handler_edit extends subs_handler_view {

  function render($values) {
    $type = $values->{$this->aliases['type']};
    $sid = $values->{$this->aliases['sid']};

    // Creating a dummy subscription to check access against.
    $dummy = (object) array('sid' => $sid, 'type' => $type);
    if (!subs_access('edit', $dummy)) {
      return;
    }

    $text = !empty($this->options['text']) ? $this->options['text'] : t('edit');

    return l($text, 'subscription/' . $sid . '/edit');
  }

}

/**
 * A handler to provide a field with a link to delete the subscription.
 *
 * @ingroup views_field_handlers
 */
class subs_handler_delete extends subs_handler_view {

  function render($values) {
    $type = $values->{$this->aliases['type']};
    $sid = $values->{$this->aliases['sid']};

    // Creating a dummy subscription to check access against.
    $dummy = (object) array('sid' => $sid, 'type' => $type);
    if (!subs_access('delete', $dummy)) {
      return;
    }

    $text = !empty($this->options['text']) ? $this->options['text'] : t('delete');

    return l($text, 'subscription/' . $sid . '/delete');
  }

}

/**
 * A handler to provide a field with operations links for the subscription.
 *
 * @ingroup views_field_handlers
 */
class subs_handler_operations extends views_handler_field {

  function construct() {
    parent::construct();
    $this->additional_fields['sid'] = 'sid';
    $this->additional_fields['type'] = 'type';
  }

  function query() {
    $this->ensure_my_table();
    $this->add_additional_fields();
  }

  function render($values) {
    $type = $values->{$this->aliases['type']};
    $sid = $values->{$this->aliases['sid']};

    $destination = drupal_get_destination();
    $operations = array();

    // Creating a dummy subscription to check access against.
    $dummy = (object) array('sid' => $sid, 'type' => $type);

    // Add view link if user has view permsission.
    if (subs_access('view', $dummy)) {
      $operations['view'] = array(
        'title' => t('view'),
        'href' => 'subscription/' . $sid,
        'destination' => $destination,
      );
    }

    // Add view link if user has edit permsission.
    if (subs_access('edit', $dummy)) {
      $operations['edit'] = array(
        'title' => t('edit'),
        'href' => 'subscription/' . $sid . '/edit',
        'destination' => $destination,
      );
    }

    // Add view link if user has delete permsission.
    if (subs_access('delete', $dummy)) {
      $operations['delete'] = array(
        'title' => t('delete'),
        'href' => 'subscription/' . $sid . '/delete',
        'destination' => $destination,
      );
    }

    return array(
      '#theme' => 'links__subs_operations',
      '#links' => $operations,
      '#attributes' => array('class' => array('links', 'inline')),
    );
  }

}