<?php
/**
 * @file
 * Drop Jobs Application pages.
 */

/**
 * Application view callback.
 * Displays an application entity.
 *
 * @see drop_jobs_application_menu()
 */
function drop_jobs_application_view($application) {
  drupal_set_title(entity_label('drop_jobs_application', $application));
  return entity_view('drop_jobs_application', array(entity_id('drop_jobs_application', $application) => $application), 'full');
}

/**
 * Ajax callback.
 * Autocompletes job titles.
 *
 * @param $string
 *  The text entered in the input field.
 * @see drop_jobs_application_menu()
 */
function drop_jobs_application_job_autocomplete($string) {
  $matches = array();

  $query = db_select('node', 'n')
            ->fields('n', array('nid', 'title'))
            ->condition('n.title', '%' . db_like($string) . '%', 'LIKE')
            ->range(0, 20);

  $return = $query->execute();

  foreach ($return as $row) {
    $matches["{$row->title} [nid:{$row->nid}]"] = check_plain($row->title);
  }

  drupal_json_output($matches);
}

/**
 * Form callback.
 *
 * Shows a confirmation form for applying to a job.
 *
 * @see drop_jobs_application_menu()
 */
function drop_jobs_application_form_apply($form, $form_state, $nid) {
  $form = array();

  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $nid,
  );

  $options = array();
  foreach (drop_jobs_application_get_types() as $type => $object) {
    $options[$type] = $object->label;
  }

  // If we only have one application type, hide the type select box.
  if (count($options) === 1) {
    $options = reset(array_keys($options));
    $form['type'] = array(
      '#type' => 'hidden',
      '#value' => $options,
    );
  }
  else {
    $form['type'] = array(
      '#type' => 'select',
      '#title' => t('Application Type'),
      '#options' => $options,
    );
  }

  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm application'),
    '#description' => t('Check the above box to confirm your application'),
    '#default_value' => FALSE,
    '#required' => TRUE,
    '#weight' => 98,
  );

  if (variable_get_value('drop_jobs_application_terms')) {
    $form['terms'] = array(
      '#type' => 'checkbox',
      '#title' => t('Terms and Conditions'),
      '#description' => t('Check the above box to confirm you read, understood and agree with our <a href="@terms">Terms and Conditions</a>', array('@terms' => url('node/5', array('absolute' => TRUE)))),
      '#default_value' => FALSE,
      '#required' => TRUE,
      '#weight' => 99,
    );
  }

  // @TODO deal with different application types.
  $application = entity_create('drop_jobs_application', array('type' => 'standard'));
  field_attach_form('drop_jobs_application', $application, $form, $form_state);

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#weight' => 100,
  );

  return $form;
}

/**
 * Validate callback.
 *
 * @see drop_jobs_application_form_apply().
 */
function drop_jobs_application_form_apply_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['nid'])) {
    form_set_error($form_state['values']['nid'], t('Invalid job.'));
  }
  if (empty($form_state['values']['type'])) {
    form_set_error($form_state['values']['type'], t('Invalid application type.'));
  }
}

/**
 * Submit callback.
 *
 * @see drop_jobs_application_form_apply().
 */
function drop_jobs_application_form_apply_submit($form, &$form_state) {
  $nid = $form_state['values']['nid'];
  $type = $form_state['values']['type'];
  $account = user_uid_optional_load();

  // Prepare values for creating new application.
  $values = array(
    'user' => $account,
    'node' => $nid,
    'type' => $type,
    'status' => DROP_JOBS_APPLICATION_ACTIVE
  );

  if ($application = drop_jobs_application_create($values)) {
    if (drop_jobs_application_save($application)) {
      // Invoke rules event.
      if (module_exists('rules')) {
        rules_invoke_event('drop_jobs_application_apply', $application);
      }

      drupal_set_message(t('You have successfully applied to this job!'));
      drupal_goto('node/' . $nid);
    }
  }

  // @TODO watchdog?
  drupal_set_message(t('An error ocurred when applying to this job, please contact the site administrators.'), 'error');
}

/**
 * Form callback.
 *
 * Shows a confirmation form for unapplying to a job.
 *
 * @see drop_jobs_application_menu().
 */
function drop_jobs_application_form_unapply($form, $form_state, $nid) {
  $form = array();

  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $nid,
  );

  $options = array();
  foreach (drop_jobs_application_get_types() as $type => $object) {
    $options[$type] = $object->label;
  }

  $account = user_uid_optional_load();
  // Load all user's applications to this job.
  $applications = drop_jobs_application_load_by_user_and_node($account, $nid);
  // Filter active applications.
  $applications = array_filter($applications, 'drop_jobs_application_filter_active');

  // If we only have one application, hide the type select box.
  if (count($applications) === 1) {
    $form['type'] = array(
      '#type' => 'hidden',
      '#value' => reset($applications)->type,
    );
  }
  else {
    $form['type'] = array(
      '#type' => 'select',
      '#title' => t('Application Type'),
      '#options' => $options,
    );
  }

  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm application removal'),
    '#description' => t('Check the above box to confirm your application removal'),
    '#default_value' => FALSE,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit')
  );

  return $form;
}

/**
 * Validate callback.
 *
 * @see drop_jobs_application_form_unapply().
 */
function drop_jobs_application_form_unapply_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['nid'])) {
    form_set_error($form_state['values']['nid'], t('Invalid job.'));
  }
  if (empty($form_state['values']['type'])) {
    form_set_error($form_state['values']['type'], t('Invalid application type.'));
  }
}

/**
 * Submit callback.
 *
 * @see drop_jobs_application_form_apply().
 */
function drop_jobs_application_form_unapply_submit($form, &$form_state) {
  $nid = $form_state['values']['nid'];
  $type = $form_state['values']['type'];
  $account = user_uid_optional_load();

  $application = drop_jobs_application_load_by_user_and_node($account, $nid, $type);

  if (is_array($application)) {
    // Filter active applications.
    $application = array_filter($application, 'drop_jobs_application_filter_active');
    $application = reset($application);
  }

  $application->status = DROP_JOBS_APPLICATION_INACTIVE;

  if (drop_jobs_application_save($application)) {
    // Invoke rules event.
    if (module_exists('rules')) {
      rules_invoke_event('drop_jobs_application_unapply', $application);
    }

    drupal_set_message(t('You have successfully removed your job application!'));
    drupal_goto('node/' . $nid);
  }
  else {
    // @TODO watchdog?
    drupal_set_message(t('An error ocurred when removing your job application, please contact the site administrators.'), 'error');
  }
}